<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
namespace FiloBlu\Esb\Observer;
use \Magento\Sales\Api\Data\OrderInterface;
use \FiloBlu\Esb\Helper\Data;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\Event\ObserverInterface;

/**
 * Observer on Order Status Update
 */
 
 class UpdateStatus implements ObserverInterface{
	
    /**
     * @var \FiloBlu\Esb\Model\ScheduleFactory
     */  	
    protected $_scheduleFactory;
    
	/**
     * @var \Psr\Log\LoggerInterface $logger
     */  
	protected $_logger;

	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \FiloBlu\Esb\Helper\Data
     */  
	protected $_helper;
	
	/**
     * @param \FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \FiloBlu\Esb\Helper\Data $helper
     */  
	public function __construct(
		\FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory, 
		\Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \FiloBlu\Esb\Helper\Data $helper
	){
		$this->_scheduleFactory=$scheduleFactory;
		$this->_logger = $logger;
        $this->_scopeConfig = $scopeConfig;
        $this->_helper = $helper;
	}

    /**
     * Send data to ESB schedule after invoice register. 
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return \Magento\Sales\Api\Data\OrderInterface 
     */
    public function execute(Observer $observer){      
        $order = $observer->getEvent()->getOrder();
        $fromStatus=$order->getOrigData("status");
        $toStatus=$order->getData("status");
        if($fromStatus==$toStatus){
            return $order;
        }
        $events=unserialize($this->_scopeConfig->getValue(
            'esb/endpoints/events', 
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        ));  
        if($events!=null && count($events)>0){
            foreach($events as $eventData){
                if($eventData["event_type"]!="order_status"){continue;}
                if($eventData["from_status"]!="*" && $eventData["from_status"]!=$fromStatus){continue;}                
                if($eventData["to_status"]!="*" && $eventData["to_status"]!=$toStatus){continue;}      
                $endpoint=$this->_helper->getEndpointUrl($eventData["event_endpoint"]);
                $schedule = $this->_scheduleFactory->create();
                $eventDescription="status_from_".$fromStatus."_to_".$toStatus;
                $scheduleData=array(
                        "object_repository_interface" => "\Magento\Sales\Api\OrderRepositoryInterface",
                        "object_id" => $order->getId(),
                        "endpoint" => $endpoint,
                        "event" => $eventDescription,
                        "status" => "pending",
                        "executable_after" => new \Zend_Db_Expr('NOW()')
                    );
                if(strlen($eventData["final_status"])>0){
                    $scheduleData["object_after_method"]="updateOrderStatus";
                    $scheduleData["object_after_method_argument"]=$eventData["final_status"];
                }
                $schedule->setData(
                    $scheduleData
                );
                $schedule->save();            
            }
		}
        return $order;
    }
}